<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/08/19
 * Time: 11.30
 */

namespace FiloBlu\Rma\Helper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Quote\Model\Quote\Address\Rate;
use Magento\Quote\Model\QuoteManagement;
use Magento\Customer\Model\CustomerFactory;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Sales\Model\Service\OrderService;
use Symfony\Component\Console\Output\ConsoleOutput as Output;
use Magento\Eav\Model\AttributeRepository;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;


class ChangeSizeOrder extends \Magento\Framework\App\Helper\AbstractHelper
{

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var QuoteManagement
     */
    protected $quoteManagement;
    /**
     * @var CustomerFactory
     */
    protected $customerFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var OrderService
     */
    protected $orderService;
    /**
     * @var Output
     */
    protected $output;
    /**
     * @var CartRepositoryInterface
     */
    protected $cartRepositoryInterface;
    /**
     * @var CartManagementInterface
     */
    protected $cartManagementInterface;
    /**
     * @var Rate
     */
    protected $shippingRate;
    /**
     * @var AttributeRepository
     */
    protected $attributeRepository;

    /**
     * @var CustomerInterfaceFactory
     */
    protected $customerInterfaceFactory;
    
    /**
     * @var StoreRepositoryInterface
     */
    protected $storeRepositoryInterface;
    
    /**
     * @var OrderItemRepositoryInterface
     */
    protected $orderItemRepositoryInterface;



    const CHANGESIZE_PAYMENT = 'changesizepay';

    const RMA_RESOLUTION_ATTRIBUTE_CODE = 'resolution';


    /**
     * ChangeSizeOrder constructor.
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param ProductRepository $productRepository
     * @param QuoteManagement $quoteManagement
     * @param CustomerFactory $customerFactory
     * @param CustomerRepositoryInterface $customerRepository
     * @param OrderService $orderService
     * @param CartRepositoryInterface $cartRepositoryInterface
     * @param CartManagementInterface $cartManagementInterface
     * @param Rate $shippingRate
     * @param Output $output
     * @param AttributeRepository $attributeRepository
     * @param CustomerInterfaceFactory $customerInterfaceFactory
     * @param StoreRepositoryInterface $storeRepositoryInterface
     * @param OrderItemRepositoryInterface $orderItemRepositoryInterface
     * @param OrderInterface $orderInterface
     */
    public function __construct(
                Context $context,
                StoreManagerInterface $storeManager,
                ProductRepository $productRepository,
                QuoteManagement $quoteManagement,
                CustomerFactory $customerFactory,
                CustomerRepositoryInterface $customerRepository,
                OrderService $orderService,
                CartRepositoryInterface $cartRepositoryInterface,
                CartManagementInterface $cartManagementInterface,
                Rate $shippingRate,
                Output $output,
                AttributeRepository $attributeRepository,
                CustomerInterfaceFactory $customerInterfaceFactory,
                StoreRepositoryInterface $storeRepositoryInterface,
                OrderItemRepositoryInterface $orderItemRepositoryInterface,
                OrderInterface $orderInterface
        ) 
    {
        $this->storeManager = $storeManager;
        $this->productRepository = $productRepository;
        $this->quoteManagement = $quoteManagement;
        $this->customerFactory = $customerFactory;
        $this->customerRepository = $customerRepository;
        $this->orderService = $orderService;
        $this->cartRepositoryInterface = $cartRepositoryInterface;
        $this->cartManagementInterface = $cartManagementInterface;
        $this->shippingRate = $shippingRate;
        $this->output = $output;
        $this->attributeRepository =$attributeRepository;
        $this->customerInterfaceFactory = $customerInterfaceFactory;
        $this->storeRepositoryInterface = $storeRepositoryInterface;
        $this->orderItemRepositoryInterface = $orderItemRepositoryInterface;
        $this->orderInterface = $orderInterface;
        parent::__construct($context);
    }


    /**
     * @param object $orderData
     * @return int|false
     */
    public function createMageOrder(array $orderData)
    {
        
        error_log(__METHOD__);
        $order = $orderData['order'];

        try
        {
            // init customer
            $customer=$this->customerFactory->create();
            
            // get original order store
            $orderStore = $order->getStoreId();
            $store = $this->storeRepositoryInterface->getById($orderStore);
            $website_id = $store->getWebsiteId();
            $customer->setWebsiteId($website_id); 
            
            // init quote
            $cart_id = $this->cartManagementInterface->createEmptyCart();
            $cart = $this->cartRepositoryInterface->get($cart_id);
            $cart->setStore($store);
            // if you have already buyer id then you can load customer directly
            if(isset($orderData['order']['customer_is_guest']) && $orderData['order']['customer_is_guest'] == "1"){
                //for guest order
                $cart->setCustomerIsGuest(true);
                $cart->setCustomerFirstName($order->getBillingAddress()->getFirstName());
                $cart->setCustomerLastname($order->getBillingAddress()->getLastName());
                $cart->setCustomerEmail($order->getCustomerEmail());
            }else{
                $_customer = $this->customerRepository->get($order->getCustomerEmail());
                $cart->assignCustomer($_customer); //Assign quote to customer
            }

            $cart->setCurrency();

            // add item to quote
            $item = $orderData['items'];
            $product = $this->productRepository->getById($item['product_id']);
            
            $order_item = $this->orderItemRepositoryInterface->get($item['original_order_item_id']);
            $final_price = ($order_item->getRowTotalInclTax() - $order_item->getBaseDiscountAmount()) / $order_item->getQtyOrdered();
            
            $product->setStoreId($orderStore)->setPrice($final_price);
            $cart->addProduct(
                $product,
                intval(1)
            );
            
            $quoteItem = $cart->getItemByProduct($product);
            $quoteItem->setOriginalPrice($final_price);
            $quoteItem->setIsSuperMode(true);
            $quoteItem->save();
            
            // Set Address to quote
            $cart->getBillingAddress()->addData( (array) $order->getBillingAddress()->getData());
            $cart->getShippingAddress()->addData( (array) $order->getShippingAddress()->getData());
            
            // Collect Rates and Set Shipping & Payment Method
            $this->shippingRate->setCode('freeshipping_freeshipping')->getPrice(0);
            $shippingAddress = $cart->getShippingAddress();
            
            $shippingAddress->setCollectShippingRates(true)
                ->collectShippingRates()
                ->setShippingMethod('freeshipping_freeshipping');
            $cart->getShippingAddress()->addShippingRate($this->shippingRate);
            $cart->setPaymentMethod(self::CHANGESIZE_PAYMENT); //payment method
            $parentSkuReplaced = $orderData['parent_product_sku'];

            $additionalInfo = [
                "parent_order_id" => $order->getId(),
                "parent_order_increment_id" => $order->getIncrementId(),
                "rma_id" => $orderData['rma_id'],
                "rma_entity_id" => $item['rma_entity_id'],
                "parent_order_item_id" => $item['original_order_item_id'],
                "parent_product_sku" => $parentSkuReplaced,
                "grand_total" => $final_price,
                "base_grand_total" => $final_price
            ];
            $cart->getPayment()->setAdditionalInformation($additionalInfo);
            $cart->getPayment()->importData(['method' => self::CHANGESIZE_PAYMENT]);
            $cart->save();
            
            // Collect total and save
            $cart->collectTotals();
            $cart->save();
            
            $order_id = $this->cartManagementInterface->placeOrder($cart->getId());
            $new_order = $this->orderInterface->load($order_id);
            $new_order->setShippingDescription('Metodo di spedizione - Standard');
            // set pending status - emmedata sync
            $new_order->setStatus('pending');
            $new_order->save();
            // srt change_size status
            $new_order->setStatus('change_size');
            $new_order->save();
            
            return $order_id;
        }
        catch (\Exception $e)
        {
            error_log($e->getMessage());
            $this->_logger->critical($e->getMessage());
            return false;
        }

    }


}